/*
 * ew_main.c
 *
 *  Created on: 21-Jul-2022
 *      Author: vmatai
 */
#include <mqx.h>
#include <bsp.h>
#include "os_functions.h"
#include "Common_defines.h"
#include "ew_main.h"
#include "Core.h"
#include "GeneratedCode/Graphics.h"
#include "_CoreView.h"

// ****************************** DeviceDriver.c *********************************************
#include "Application.h"

#ifdef _ApplicationDeviceClass_

/*
   Create a static variable to keep the global instance (autoobject) of the
   device class. The type of the variable has to correspond to the device class
   and the unit name, for example the static variable of the class 'DeviceClass'
   from the unit 'Application' has the type 'ApplicationDeviceClass'.
*/
  static ApplicationDeviceClass DeviceObject = 0;
  static float Xcoord=0.0, Ycoord=0.0;

#endif

  boolean clear_EW = FALSE;
static CoreRoot     RootObject;
static XViewport*   Viewport;
static uint_8 count = 0;

#define CHECK_HANDLE( handle ) \
  if ( !handle )               \
  {                            \
	  printf(" [failed]\n" );   \
    return 0;                  \
  }                            \
  else                         \
  printf(" [OK]\n" );

// ****************************** ew_bsp_clock.c starts ********************************************
/* timer tick for Embedded Wizard UI applications */
static volatile unsigned long EmWiSystemTicks = 0;

void EwBspClockTickIncrement( void )
{
  EmWiSystemTicks++;
}

unsigned long EwBspClockGetTicks( void )
{
//			MQX_TICK_STRUCT			tick_start;
//		  _time_get_elapsed_ticks(&tick_start);
//		  EmWiSystemTicks = tick_start.HW_TICKS;
  return EmWiSystemTicks;
}

#if EW_CPU_LOAD_MEASURING == 1

  /* variables for cycle counter / CPU load measurement */
  static unsigned long          WorkingTime  = 0;
  static unsigned long          SleepingTime = 0;
  static unsigned long          PrevCycCnt   = 0;


  /* helper function to get cycle counter value since last call */
  static unsigned long GetCycleCounterDelta( void )
  {
    unsigned long cycCnt;
    unsigned long result;

    /* get delta between current counter value and last counter value */
    cycCnt = DWT->CYCCNT;
    result = cycCnt - PrevCycCnt;
    PrevCycCnt = cycCnt;

    return result;
  }

#endif

 void EwBspClockCpuLoadSetIdle( void )
{
  #if EW_CPU_LOAD_MEASURING == 1

    WorkingTime += GetCycleCounterDelta();

  #endif
}

 void EwBspClockCpuLoadSetActive( void )
 {
   #if EW_CPU_LOAD_MEASURING == 1

     SleepingTime += GetCycleCounterDelta();

   #endif
 }

// ****************************** ew_bsp_console.c *********************************************
 void EwBspConsoleInit( void )
 {
 }
 void EwBspConsolePutCharacter( unsigned char aCharacter )
 {
   //DbgConsole_Putchar( aCharacter );
 }
 unsigned char* EwBspConsoleGetCharacter( void )
 {
   return 0;
 }

// ****************************** ew_bsp_display.c *********************************************

#if EW_USE_DOUBLE_BUFFER == 1

  static __IO uint32_t          PendingFramebuffer = 0;

#else

  static volatile unsigned long VSyncTime          = 0;
  static volatile unsigned long VSyncDuration      = 0;

#endif

int EwBspDisplayInit( int aGuiWidth, int aGuiHeight, XDisplayInfo* aDisplayInfo )
{
	EW_UNUSED_ARG( aGuiWidth );
	  EW_UNUSED_ARG( aGuiHeight );

	  // check and clean display info structure
	  if ( !aDisplayInfo )
	    return 0;
	  memset( aDisplayInfo, 0, sizeof( XDisplayInfo ));

	  aDisplayInfo->FrameBuffer    = (void*)os_mem_alloc(EW_FRAME_BUFFER_SIZE);	// EW_FRAME_BUFFER_ADDR;
	  aDisplayInfo->DoubleBuffer   = (void*)os_mem_alloc(EW_DOUBLE_BUFFER_SIZE);	// EW_DOUBLE_BUFFER_ADDR;
	  aDisplayInfo->BufferWidth    = EW_FRAME_BUFFER_WIDTH;
	  aDisplayInfo->BufferHeight   = EW_FRAME_BUFFER_HEIGHT;
	  aDisplayInfo->DisplayWidth   = EW_DISPLAY_WIDTH;
	  aDisplayInfo->DisplayHeight  = EW_DISPLAY_HEIGHT;
	  aDisplayInfo->MemoryPool    = (void*)os_mem_alloc(EW_MEMORY_POOL_SIZE);	// EW_MEMORY_POOL_SIZE;

	  #if EW_USE_DOUBLE_BUFFER == 1
	    aDisplayInfo->UpdateMode   = EW_BSP_DISPLAY_UPDATE_NORMAL;
	  #else
	    aDisplayInfo->UpdateMode   = EW_BSP_DISPLAY_UPDATE_PARTIAL;
	  #endif

	  /*
	   #if EW_USE_FREE_RTOS == 1

			//create semaphore for handling double-buffering on V-sync
			LcdUpdateSemaphore = xSemaphoreCreateBinary();
			xSemaphoreGive( LcdUpdateSemaphore );

		 #endif
	  BOARD_InitLcdifPixelClock();
	  BOARD_InitLcd();

	  */
	  // ******************************   fsl_elcdif.h ********************************************
	    // ******************************   imxrt_lcd.h********************************************
	  const elcdif_rgb_mode_config_t config =
	  {
	    .panelWidth = EW_FRAME_BUFFER_WIDTH,
	    .panelHeight = EW_FRAME_BUFFER_HEIGHT,
	    .hsw = LCD_HSW,
	    .hfp = LCD_HFP,
	    .hbp = LCD_HBP,
	    .vsw = LCD_VSW,
	    .vfp = LCD_VFP,
	    .vbp = LCD_VBP,
	    .polarityFlags = LCD_POL_FLAGS,
	    .bufferAddr = ( uint32_t ) aDisplayInfo->FrameBuffer,
	    .pixelFormat = kELCDIF_PixelFormatRGB565,
	    .dataBus = kELCDIF_DataBus16Bit

	  };

	  ELCDIF_RgbModeInit(LCDIF, &config);

	//  EnableIRQ(LCDIF_IRQn);
	//  NVIC_SetPriority(LCDIF_IRQn, 8);
	//  ELCDIF_EnableInterrupts(LCDIF, kELCDIF_CurFrameDoneInterruptEnable);

	  ELCDIF_RgbModeStart(LCDIF);

	  // return the current display configuration

	  return 1;
}

void EwBspDisplayDone( XDisplayInfo* aDisplayInfo )
{
  EW_UNUSED_ARG( aDisplayInfo );
}

int EwBspDisplayGetUpdateArea( XRect* aUpdateRect )
{
  #if EW_USE_DOUBLE_BUFFER == 0

    static int field = 0;

    /* determine rectangular area of current field */
    #if EW_SURFACE_ROTATION == 0
      *aUpdateRect = EwNewRect( 0, field * EW_FRAME_BUFFER_HEIGHT / EW_NUMBER_OF_FIELDS,
        EW_FRAME_BUFFER_WIDTH, ( field + 1 ) * EW_FRAME_BUFFER_HEIGHT / EW_NUMBER_OF_FIELDS );
    #endif

    #if EW_SURFACE_ROTATION == 90
      *aUpdateRect = EwNewRect( field * EW_FRAME_BUFFER_HEIGHT / EW_NUMBER_OF_FIELDS, 0,
        ( field + 1 ) * EW_FRAME_BUFFER_HEIGHT / EW_NUMBER_OF_FIELDS, EW_FRAME_BUFFER_WIDTH );
    #endif

    #if EW_SURFACE_ROTATION == 180
      *aUpdateRect = EwNewRect( 0, EW_FRAME_BUFFER_HEIGHT - ( field + 1 ) * EW_FRAME_BUFFER_HEIGHT / EW_NUMBER_OF_FIELDS,
        EW_FRAME_BUFFER_WIDTH, EW_FRAME_BUFFER_HEIGHT - field * EW_FRAME_BUFFER_HEIGHT / EW_NUMBER_OF_FIELDS );
    #endif

    #if EW_SURFACE_ROTATION == 270
      *aUpdateRect = EwNewRect( EW_FRAME_BUFFER_HEIGHT - ( field + 1 ) * EW_FRAME_BUFFER_HEIGHT / EW_NUMBER_OF_FIELDS,
        0, EW_FRAME_BUFFER_HEIGHT - field * EW_FRAME_BUFFER_HEIGHT / EW_NUMBER_OF_FIELDS, EW_FRAME_BUFFER_WIDTH );
    #endif

    /* next field */
    field++;

    /* sync on start line of next field to ensure save drawing operation */
    SyncOnLine(( field % EW_NUMBER_OF_FIELDS ) * EW_FRAME_BUFFER_HEIGHT / EW_NUMBER_OF_FIELDS );

    if ( field <= EW_NUMBER_OF_FIELDS )
      return 1;

    field = 0;

  #endif

  return 0;
}

void EwBspDisplayWaitForCompletion( void )
{
  #if EW_USE_DOUBLE_BUFFER == 1

    CPU_LOAD_SET_IDLE();		//leads to ew_bsp_clock; VM

    #if EW_USE_FREE_RTOS == 1

      /* wait until pending framebuffer is used as current framebuffer and
         use CPU time for other tasks */
      while ( PendingFramebuffer )
      {
        if ( xSemaphoreTake( LcdUpdateSemaphore, 1000 / portTICK_PERIOD_MS ) == pdTRUE )
          xSemaphoreGive( LcdUpdateSemaphore );
      }

    #else

      /* wait until pending framebuffer is used as current framebuffer */
//      while( PendingFramebuffer )
//        ;

    #endif

    CPU_LOAD_SET_ACTIVE();	//leads to ew_bsp_clock; VM

  #endif
}

void EwBspDisplayCommitBuffer( void* aAddress, int aX, int aY, int aWidth, int aHeight )
{
  #if EW_USE_DOUBLE_BUFFER == 1

    DisableIRQ( LCDIF_IRQn );

    /* set pending framebuffer address to be used on next V-sync */
    PendingFramebuffer = (uint32_t)aAddress;
    ELCDIF_SetNextBufferAddr( LCDIF, (uint32_t)aAddress );

    #if EW_USE_FREE_RTOS == 1
      xSemaphoreTake( LcdUpdateSemaphore, 0 );
    #endif

    EnableIRQ( LCDIF_IRQn );

  #endif
}

void LCDIF_IRQHandler(void)
{

	  uint32_t intStatus;

	  intStatus = ELCDIF_GetInterruptStatus( LCDIF );
	  ELCDIF_ClearInterruptStatus( LCDIF, intStatus );

	  if ( intStatus & kELCDIF_CurFrameDone )
	  {
	    #if EW_USE_DOUBLE_BUFFER == 1

	      if ( PendingFramebuffer )
	      {
	        #if EW_USE_FREE_RTOS == 1

	          xSemaphoreGiveFromISR( LcdUpdateSemaphore, NULL );

	        #endif

	        PendingFramebuffer = 0;
	      }

	    #else

	      unsigned long curTicks = DWT->CYCCNT;
	      VSyncDuration = curTicks - VSyncTime;
	      VSyncTime = curTicks;

	    #endif
	  }

	  /* add for ARM errata 838869, affects Cortex-M4, Cortex-M4F Store immediate overlapping
	     exception return operation might vector to incorrect interrupt */
	  #if defined __CORTEX_M && (__CORTEX_M == 4U)
	    __DSB();
	  #endif


}

void EwBspDisplaySetClut( unsigned long* aClut )
{
  int i;
  static uint32_t clut[ 256 ];

  /* preparing the CLUT in 16bit format in order to fit to LCDIF configuration and
     the connected display */
  for ( i = 0; i < 256; i++ )
    clut[ i ] = ((( aClut[ i ] & 0x00F80000 ) >> ( 3 + 16 )) << 11 ) |
                ((( aClut[ i ] & 0x0000FC00 ) >> ( 2 +  8 )) <<  5 ) |
                ((( aClut[ i ] & 0x000000F8 ) >> ( 3 +  0 )) <<  0 );

  /* load the LUT data */
  ELCDIF_UpdateLut( LCDIF, kELCDIF_Lut0, 0, clut, ELCDIF_LUT_ENTRY_NUM );
  ELCDIF_UpdateLut( LCDIF, kELCDIF_Lut1, 0, clut, ELCDIF_LUT_ENTRY_NUM );

  ELCDIF_EnableLut( LCDIF, true );
}

/************************************ DeviceDriver.c *******************************************/
void DeviceDriver_Initialize( void )
{
  /*
     You can implement here the necessary code to initialize your particular
     hardware, to open needed devices, to open communication channels, etc.
  */

  /* configure LED */
  //EwBspInOutInitLed();

  /* Configure interrupt for hardware button */
  //EwBspInOutInitButton( HardButtonIsrCallback );

#ifdef _ApplicationDeviceClass_

  /*
     Get access to the counterpart of this device driver: get access to the
     device class that is created as autoobject within your Embedded Wizard
     project. For this purpose you can call the function EwGetAutoObject().
     This function contains two paramters: The pointer to the autoobject and
     the class of the autoobject.
     Assuming you have implemented the class 'DeviceClass' within the unit
     'Application' and you have an autoobject with the name 'Device', make
     the following call:
     EwGetAutoObject( &ApplicationDevice, ApplicationDeviceClass );
  */

  DeviceObject = EwGetAutoObject( &ApplicationDevice, ApplicationDeviceClass );

  /*
     Once you have the access to the autoobject, lock it as long as you need
     the access (typically, until your device driver is closed). By locking
     the autoobject, you can ensure that the object of the device class will
     be kept within the memory and not freed by the Garbage Collector.
     Once the device class is locked, you can easily store it within a static
     variable to access the driver class during the runtime.
  */

  EwLockObject( DeviceObject );

#endif

}

void updateProg_Counter(float xc, float yc){
	Xcoord = xc*100;
	Ycoord = yc*100000;
	printf("\nIn updateProg_Counter;\n");
	printf("Xcoord = %f, Ycoord = %f\n", Xcoord,Ycoord);
}

int DeviceDriver_ProcessData( void )
{
  int needUpdate = 0;

  /*
     Get the data you want to provide to the GUI application.
     In case your are working with an operating system and your device is
     controlled from a separate task/thread/process, take all information
     from your device driver out of the message queue.
     Please note, that this function is called within the context of the main
     GUI thread.
     If you control your system by direct register access or some BSP functions,
     get all necessary data you want to provide to the GUI application.
  */

#ifdef _ApplicationDeviceClass_

	#ifdef _ApplicationDeviceClass__UpdateX_

    ApplicationDeviceClass__UpdateX( DeviceObject, Xcoord );
    needUpdate = 1;

   #endif

	#ifdef _ApplicationDeviceClass__UpdateY_

    ApplicationDeviceClass__UpdateY( DeviceObject, Ycoord );
	 needUpdate = 1;

	#endif

	#ifdef _ApplicationDeviceClass__UpdateProg_property_

	 ApplicationDeviceClass__UpdateProg_property( DeviceObject, (XString)emwiString );
	 needUpdate = 1;

	#endif

	#ifdef _ApplicationDeviceClass__UpdateTerminalString_

    ApplicationDeviceClass__UpdateTerminalString( DeviceObject, (XString)str );
    needUpdate = 1;
    //printf("\nReceived string = %c", ch);

    #endif

	 /* When a character is received from the serial terminal, call the method
	      'UpdateTerminalString' of the device class 'DeviceClass' within the unit
	      'Application' and provide the character */
/*    ch = EwBspConsoleGetCharacter();
	   if (( ch >= 'A' ) && ( ch <= 'Z' ) || ( ch >= 'a' ) && ( ch <= 'z' )
	     || ( ch >= '0' ) && ( ch <= '9' ) || ( ch == ' ' ))
	   {
	     ApplicationDeviceClass__UpdateTerminalString( DeviceObject, (XChar)ch );
	     needUpdate = 1;
	     printf("\nReceived string = %c", ch);
	   }
*/
  /* here we just evaluate the current hardware button state */
  //if ( IsHardButtonDown )
  //  ButtonCounter++;

  /* check for a valid access to the autoobject of the device class */
  if ( DeviceObject == 0 )
    return 0;

  /*
     For each device paramter, that is represented by a property within the
     Embedded Wizard device class and that you want to update, you have to call
     the appropriate UpdateProperty() method.

     The following examples assumes, that you have a device class with the
     name 'DeviceClass' within the unit 'Application'.
  */

  /* Update the property HardButtonCounter within the class Application::DeviceClass
     by calling the method 'UpdateHardButtonCounter' - the generated define is
     evaluated to ensures that the method is available within the generated code. */
/*  if ( ButtonCounter > 0 )
  {
    // clear counter if button is released
    //if ( !IsHardButtonDown )
      //ButtonCounter = 0;

    #ifdef _ApplicationDeviceClass__UpdateHardButtonCounter_

      ApplicationDeviceClass__UpdateHardButtonCounter( DeviceObject, (XInt32)ButtonCounter );
      needUpdate = 1;

    #endif
  }
*/
  /*
     Trigger system events if necessary, e.g. if a certain situation happens,
     if an error occurs or just if a certain value has changed...
  */

  /* When the hardware button is pressed, call the method 'TriggerHardButtonEvent()' of the
     device class 'DeviceClass' within the unit 'Application' - the generated define is
     evaluated to ensure that the method is available within the generated code. */
/*  if ( ButtonCounter == 1 )
  {
    #ifdef _ApplicationDeviceClass__TriggerHardButtonEvent_

      ApplicationDeviceClass__TriggerHardButtonEvent( DeviceObject );
      needUpdate = 1;

    #endif
  }
*/
#endif

  /*
     Return a value != 0 if there is at least one property changed or if a
     system event was triggered. The return value is used by the main loop, to
     decide whether the GUI application has changed or not.
  */

  return needUpdate;
}
void DeviceDriver_Deinitialize( void )
{
  /*
     You can implement here the necessary code to deinitialize your particular
     hardware, to close used devices, to close communication channels, etc.
  */

#ifdef _ApplicationDeviceClass_

  /*
     If you have access to the device class autoobject, don't forget to unlock
     it. Clear the static variable to avoid later usage.
  */

  if ( DeviceObject )
    EwUnlockObject( DeviceObject );

  DeviceObject = 0;

#endif

}
void DeviceDriver_PrintMessage( XString aText )
{
  /* just print the given 16bit string... */
  printf( "The string is: %S\n", aText );
}

void DeviceDriver_SetLedStatus( XInt32 aValue )
{
  /*
     In case you are using an operating system to communicate with your
     device driver that is running within its own thread/task/process,
     send a message to the device driver and transmit the new value.
     Please note, that this function is called within the context of the main
     GUI thread.
  */

  /*
     Here we are accessing directly the device driver by calling a certain
     BSP / driver function.
  */
/*
  if ( aValue )
    EwBspInOutLedOn();
  else
    EwBspInOutLedOff();
*/
}
//*******************************************************************************
// ****************************** ew_bsp_graphics.c *********************************************

#if EW_USE_FREE_RTOS == 1

  #include "FreeRTOS.h"
  #include "task.h"
  #include "semphr.h"

  static SemaphoreHandle_t    AcceleratorSemaphore;

#endif


static volatile char       TransferInProgress  = 0;
static char                ConcurrentOperation = 1;

static pxp_output_buffer_config_t OutputBufferConfig;
static pxp_ps_buffer_config_t     ProcessSurfaceConfig;
static pxp_as_buffer_config_t     AlphaSurfaceConfig;
static pxp_as_blend_config_t      BlendConfig;

int EwBspGraphicsInit( void )
{
  #if EW_USE_FREE_RTOS == 1

    /* Create the accelerator semaphore */
    AcceleratorSemaphore = xSemaphoreCreateBinary();

  #endif

  /* initialize the PXP graphics accelerator */
  PXP_Init( PXP );

  PXP_EnableAlphaSurfaceOverlayColorKey( PXP, false );
  PXP_EnableOverWrittenAlpha( PXP, false );

  /* disable CSC1 */
  PXP_EnableCsc1( PXP, false );

  #ifdef EW_USE_PXP_INTERRUPT_MODE

    /* enable interrupt for PXP completion */
    EnableIRQ( PXP_IRQn );
    NVIC_SetPriority( PXP_IRQn, 8 );
    PXP_EnableInterrupts( PXP, kPXP_CompleteInterruptEnable );

  #endif

  return 1;
}
/*******************************************************************************
* FUNCTION:
*   EwBspGraphicsDone
*
* DESCRIPTION:
*   Deinitialize the PXP graphics accelerator.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspGraphicsDone( void )
{
  /* deinitialize the PXP graphics accelerator */
  PXP_Deinit( PXP );
}


/*******************************************************************************
* FUNCTION:
*   EwBspGraphicsWaitForCompletion
*
* DESCRIPTION:
*   The function EwBspGraphicsWaitForCompletion returns as soon as the PXP has
*   completed a pending graphics instruction. If the PXP is in idle mode,
*   the function returns immediately.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspGraphicsWaitForCompletion()
{
  /* return immediately if no PXP transfer is ongoing */
  if ( TransferInProgress == 0 )
    return;

  CPU_LOAD_SET_IDLE();		//VM


#ifdef EW_USE_PXP_INTERRUPT_MODE

  #if EW_USE_FREE_RTOS == 1

    xSemaphoreTake( AcceleratorSemaphore, 1000 / portTICK_PERIOD_MS );
    TransferInProgress = 0;

  #else

    /* wait until PXP transfer is done */
    while( TransferInProgress )
      ;

  #endif

#else

  /* wait until PXP transfer is done */
  while (!(PXP_GetStatusFlags( PXP ) & kPXP_CompleteFlag ))
    ;

  PXP_ClearStatusFlags( PXP, kPXP_CompleteFlag );

  TransferInProgress = 0;

#endif

  CPU_LOAD_SET_ACTIVE();		//VM
}


/*******************************************************************************
* FUNCTION:
*   EwBspGraphicsConcurrentOperation
*
* DESCRIPTION:
*   The function EwBspGraphicsConcurrentOperation configures the operation mode
*   of PXP and CPU. If concurrent operation is enabled, the CPU will work in
*   parallel while the PXP is transferring data. If concurrent operation is
*   disabled, the CPU will wait everytime the PXP is active.
*   This feature is intended to limit the memory bandwidth, e.g. during display
*   update or other bandwidth consuming activities.
*
* ARGUMENTS:
*   aEnable - flag to switch on/off concurrent operation mode.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspGraphicsConcurrentOperation( int aEnable )
{
  ConcurrentOperation = (char)aEnable;
}


/*******************************************************************************
* FUNCTION:
*   EwBspGraphicsFill
*
* DESCRIPTION:
*   The function EwBspGraphicsFill is used by the Graphics Engine, when a
*   rectangular area should be filled with a constant color by using the PXP
*   functionality.
*
* ARGUMENTS:
*   aDstAddr      - Destination address of the transfer.
*   aDstPitch     - Number of bytes per line within the destination bitmap.
*   aWidth,
*   aHeight       - Size of the area to fill.
*   aDstColorMode - Colorformat of the destination.
*   aSrcColor     - Color to fill the rectangular area.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspGraphicsFill( uint32_t aDstAddr, uint32_t aDstPitch, uint32_t aWidth,
  uint32_t aHeight, uint32_t aDstColorMode, uint32_t aSrcColor )
{
  /* ensure that previous PXP instruction is finished */
  EwBspGraphicsWaitForCompletion();

  /* configure alpha surface settings */
  PXP_SetAlphaSurfacePosition( PXP, 1, 1, 0, 0 );

  /* configure process surface settings - background color does not support alpha */
  PXP_SetProcessSurfaceBackGroundColor( PXP, aSrcColor );
  PXP_SetProcessSurfacePosition(  PXP, 1, 1, 0, 0 );

  /* configure output surface settings */
  OutputBufferConfig.pixelFormat    = (pxp_output_pixel_format_t)aDstColorMode;
  OutputBufferConfig.interlacedMode = kPXP_OutputProgressive;
  OutputBufferConfig.buffer0Addr    = aDstAddr;
  OutputBufferConfig.buffer1Addr    = 0;
  OutputBufferConfig.pitchBytes     = aDstPitch;
  OutputBufferConfig.width          = aWidth;
  OutputBufferConfig.height         = aHeight;

  PXP_SetOutputBufferConfig( PXP, &OutputBufferConfig );

  /* set the flag for PXP transfer */
  TransferInProgress = 1;

  /* start the transfer */
  PXP_Start( PXP );

  /* check for immediate completion of drawing operation */
  if ( !ConcurrentOperation )
    EwBspGraphicsWaitForCompletion();
}


/*******************************************************************************
* FUNCTION:
*   EwBspGraphicsCopy
*
* DESCRIPTION:
*   The function EwBspGraphicsCopy is used by the Graphics Engine, when a source
*   bitmap should be copied into a destination bitmap by using the PXP
*   functionality.
*
* ARGUMENTS:
*   aDstAddr      - Destination address of the transfer.
*   aSrcAddr      - Source address of the transfer.
*   aDstPitch     - Number of bytes per line within the destination bitmap.
*   aSrcPitch     - Number of bytes per line within the source bitmap.
*   aWidth,
*   aHeight       - Size of the area to transfer the bitmap.
*   aDstColorMode - Colorformat of the destination bitmap
*   aSrcColorMode - Colorformat of the source bitmap.
*   aSrcColor     - Color/alpha value to be used for the transfer.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspGraphicsCopy( uint32_t aDstAddr, uint32_t aSrcAddr, uint32_t aDstPitch,
  uint32_t aSrcPitch, uint32_t aWidth, uint32_t aHeight, uint32_t aDstColorMode,
  uint32_t aSrcColorMode, uint32_t aSrcColor )
{
  unsigned char alpha = (unsigned char)( aSrcColor >> 24 );

  /* ensure that previous PXP instruction is finished */
  EwBspGraphicsWaitForCompletion();

  /* in case of any Y8/YUV source color format the process surface has to be used */
  if ( aSrcColorMode & 0x10 )
  {
    BlendConfig.alpha       = 0x00;
    BlendConfig.invertAlpha = false;
    BlendConfig.alphaMode   = kPXP_AlphaOverride;
    BlendConfig.ropMode     = kPXP_RopMaskAs;

    /* configure alpha surface settings */
    PXP_SetAlphaSurfaceBlendConfig( PXP, &BlendConfig );
    PXP_SetAlphaSurfacePosition( PXP, 1, 1, 0, 0 );

    /* configure process surface settings - it does not support alpha */
    ProcessSurfaceConfig.pixelFormat = (pxp_ps_pixel_format_t)aSrcColorMode;
    ProcessSurfaceConfig.swapByte    = 0;
    ProcessSurfaceConfig.bufferAddr  = aSrcAddr;
    ProcessSurfaceConfig.bufferAddrU = 0;
    ProcessSurfaceConfig.bufferAddrV = 0;
    ProcessSurfaceConfig.pitchBytes  = aSrcPitch;

    PXP_SetProcessSurfaceBufferConfig( PXP, &ProcessSurfaceConfig );
    PXP_SetProcessSurfacePosition( PXP, 0, 0, aWidth, aHeight );
  }
  else
  {
    /* configure alpha surface settings */
    AlphaSurfaceConfig.pixelFormat = (pxp_as_pixel_format_t)aSrcColorMode;
    AlphaSurfaceConfig.bufferAddr  = aSrcAddr;
    AlphaSurfaceConfig.pitchBytes  = aSrcPitch;

    BlendConfig.alpha       = alpha;
    BlendConfig.invertAlpha = false;
    BlendConfig.alphaMode   = (alpha == 0xFF) ? kPXP_AlphaEmbedded : kPXP_AlphaMultiply;
    BlendConfig.ropMode     = kPXP_RopMaskAs;

    PXP_SetAlphaSurfaceBufferConfig( PXP, &AlphaSurfaceConfig );
    PXP_SetAlphaSurfaceBlendConfig( PXP, &BlendConfig );
    PXP_SetAlphaSurfacePosition( PXP, 0, 0, aWidth, aHeight );

    /* configure process surface settings */
    PXP_SetProcessSurfaceBackGroundColor( PXP, 0x00000000 );
    PXP_SetProcessSurfacePosition( PXP, 1, 1, 0, 0 );
  }

  /* configure output surface settings */
  OutputBufferConfig.pixelFormat    = (pxp_output_pixel_format_t)aDstColorMode;
  OutputBufferConfig.interlacedMode = kPXP_OutputProgressive;
  OutputBufferConfig.buffer0Addr    = aDstAddr;
  OutputBufferConfig.buffer1Addr    = 0;
  OutputBufferConfig.pitchBytes     = aDstPitch;
  OutputBufferConfig.width          = aWidth;
  OutputBufferConfig.height         = aHeight;

  PXP_SetOutputBufferConfig( PXP, &OutputBufferConfig );

  /* set the flag for PXP transfer */
  TransferInProgress = 1;

  /* start the transfer */
  PXP_Start( PXP );

  /* check for immediate completion of drawing operation */
  if ( !ConcurrentOperation )
    EwBspGraphicsWaitForCompletion();
}


/*******************************************************************************
* FUNCTION:
*   EwBspGraphicsCopyBlend
*
* DESCRIPTION:
*   The function EwBspGraphicsCopyBlend is used by the Graphics Engine, when a
*   source bitmap should be blended over a destination bitmap by using the PXP
*   functionality.
*
* ARGUMENTS:
*   aDstAddr      - Destination address of the transfer.
*   aSrcAddr      - Source address of the transfer.
*   aDstPitch     - Number of bytes per line within the destination bitmap.
*   aSrcPitch     - Number of bytes per line within the source bitmap.
*   aWidth,
*   aHeight       - Size of the area to transfer the bitmap.
*   aDstColorMode - Colorformat of the destination bitmap
*   aSrcColorMode - Colorformat of the source bitmap.
*   aSrcColor     - Color/alpha value to be used for the transfer.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspGraphicsCopyBlend( uint32_t aDstAddr, uint32_t aSrcAddr, uint32_t aDstPitch,
  uint32_t aSrcPitch, uint32_t aWidth, uint32_t aHeight, uint32_t aDstColorMode,
  uint32_t aSrcColorMode, uint32_t aSrcColor )
{
  unsigned char alpha = (unsigned char)( aSrcColor >> 24 );

  /* ensure that previous PXP instruction is finished */
  EwBspGraphicsWaitForCompletion();

  /* configure alpha surface settings */
  AlphaSurfaceConfig.pixelFormat = (pxp_as_pixel_format_t)aSrcColorMode;
  AlphaSurfaceConfig.bufferAddr  = aSrcAddr;
  AlphaSurfaceConfig.pitchBytes  = aSrcPitch;

  BlendConfig.alpha       = alpha;
  BlendConfig.invertAlpha = false;
  BlendConfig.alphaMode   = (alpha == 0xFF) ? kPXP_AlphaEmbedded : kPXP_AlphaMultiply;
  BlendConfig.ropMode     = kPXP_RopMaskAs;

  PXP_SetAlphaSurfaceBufferConfig( PXP, &AlphaSurfaceConfig );
  PXP_SetAlphaSurfaceBlendConfig( PXP, &BlendConfig );
  PXP_SetAlphaSurfacePosition( PXP, 0, 0, aWidth, aHeight );

  /* configure process surface settings -
     attention: it does not support alpha and does not support packed RGB888 */
  ProcessSurfaceConfig.pixelFormat = (pxp_ps_pixel_format_t)aDstColorMode;
  ProcessSurfaceConfig.swapByte    = 0;
  ProcessSurfaceConfig.bufferAddr  = aDstAddr;
  ProcessSurfaceConfig.bufferAddrU = 0;
  ProcessSurfaceConfig.bufferAddrV = 0;
  ProcessSurfaceConfig.pitchBytes  = aDstPitch;

  PXP_SetProcessSurfaceBufferConfig( PXP, &ProcessSurfaceConfig );
  PXP_SetProcessSurfacePosition( PXP, 0, 0, aWidth, aHeight );

  /* configure output surface settings */
  OutputBufferConfig.pixelFormat    = (pxp_output_pixel_format_t)aDstColorMode;
  OutputBufferConfig.interlacedMode = kPXP_OutputProgressive;
  OutputBufferConfig.buffer0Addr    = aDstAddr;
  OutputBufferConfig.buffer1Addr    = 0;
  OutputBufferConfig.pitchBytes     = aDstPitch;
  OutputBufferConfig.width          = aWidth;
  OutputBufferConfig.height         = aHeight;

  PXP_SetOutputBufferConfig( PXP, &OutputBufferConfig );

  /* set the flag for PXP transfer */
  TransferInProgress = 1;

  /* start the transfer */
  PXP_Start( PXP );

  /* check for immediate completion of drawing operation */
  if ( !ConcurrentOperation )
    EwBspGraphicsWaitForCompletion();
}


#ifdef EW_USE_PXP_INTERRUPT_MODE

/*******************************************************************************
* FUNCTION:
*   PXP_IRQHandler
*
* DESCRIPTION:
*   PXP Interrupt Handler.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void PXP_IRQHandler( void )
{
  if (PXP_GetStatusFlags( PXP ) & kPXP_CompleteFlag )
  {
    PXP_ClearStatusFlags( PXP, kPXP_CompleteFlag );

    #if EW_USE_FREE_RTOS == 1

      xSemaphoreGiveFromISR( AcceleratorSemaphore, NULL );

    #else

      TransferInProgress = 0;

    #endif
  }
}

#endif
//*******************************************************************************

// ****************************** ewmain.c **************************************
/*******************************************************************************
* FUNCTION:
*   EwGetKeyCommand
*
* DESCRIPTION:
*   The function EwGetKeyCommand reads the next key code from the console and
*   translates it into an Embedded Wizard key code. The mapping between the key
*   code from the console and the resulting Embedded Wizard key code can be
*   adapted to the needs of your application.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   Returns the next EmWi key code or CoreKeyCodeNoKey if no key code available.
*
*******************************************************************************/
static XEnum EwGetKeyCommand( void )
{
  #if EW_USE_TERMINAL_INPUT == 1
    switch ( EwBspConsoleGetCharacter())
    {
      case 0x65 : printf("Key 'Exit' pressed\n");  return CoreKeyCodeExit;
      case 0x38 : printf("Key 'Up' pressed\n");    return CoreKeyCodeUp;
      case 0x32 : printf("Key 'Down' pressed\n");  return CoreKeyCodeDown;
      case 0x36 : printf("Key 'Right' pressed\n"); return CoreKeyCodeRight;
      case 0x34 : printf("Key 'Left' pressed\n");  return CoreKeyCodeLeft;
      case 0x35 : printf("Key 'OK' pressed\n");    return CoreKeyCodeOk;
      case 0x6D : printf("Key 'Menu' pressed\n");  return CoreKeyCodeMenu;
      case 0x70 : printf("Key 'Power' pressed\n"); return CoreKeyCodePower;
    }
  #endif
  return CoreKeyCodeNoKey;
}

static void EwUpdate( XViewport* aViewport, CoreRoot aApplication )
{
	//printf("\n In EwUpdate; Count = %d \n", count++);
 XBitmap*       bitmap;
  GraphicsCanvas canvas = EwNewObject( GraphicsCanvas, 0 );
  XRect          updateRect = {{ 0, 0 }, { 0, 0 }};

  if ( !canvas )
    return;

  if ( DisplayInfo.UpdateMode == EW_BSP_DISPLAY_UPDATE_NORMAL )
  {
    bitmap = EwBeginUpdate( aViewport );

    // redraw the dirty area of the screen
    if ( bitmap  )
    {
      GraphicsCanvas__AttachBitmap( canvas, (XHandle)bitmap );
      updateRect = CoreRoot__UpdateGE20( aApplication, canvas );
      GraphicsCanvas__DetachBitmap( canvas );
      EwEndUpdate( aViewport, updateRect );
    }
  }
  else
  {
    int regions = CoreRoot__BeginUpdate( aApplication );

    while ( regions-- )
    {
      // get rectangular area of the update region for scratch-pad buffer
      if ( DisplayInfo.UpdateMode == EW_BSP_DISPLAY_UPDATE_SCRATCHPAD )
        updateRect = CoreRoot__GetUpdateRegion( aApplication, regions );

      // iterate through all update areas
      while ( EwBspDisplayGetUpdateArea( &updateRect ))
      {
        // update the current subarea
        bitmap = EwBeginUpdateArea( aViewport, updateRect );
        GraphicsCanvas__AttachBitmap( canvas, (XHandle)bitmap );
        CoreRoot__UpdateCanvas( aApplication, canvas, updateRect.Point1 );
        GraphicsCanvas__DetachBitmap( canvas );
        EwEndUpdate( aViewport, updateRect );
      }

      if ( DisplayInfo.UpdateMode != EW_BSP_DISPLAY_UPDATE_SCRATCHPAD )
        break;
    }
    CoreRoot__EndUpdate( aApplication );
  }

}


int EwInit( void )
{
	/* initalize system clocks
	  EwBspClockInit();

	  // set RTC, if current RTC time is before the minimum time
	  if ( EwBspClockGetTime() < RTC_MINIMUM_TIME )
	    EwBspClockSetTime( RTC_DEFAULT_TIME );
	*/
	EmWiSystemTicks = 0;
	// initialize display
	printf("\n Initialize Display...           ");
   CHECK_HANDLE( EwBspDisplayInit( EwScreenSize.X, EwScreenSize.Y, &DisplayInfo ));
	/*
  // initialize touchscreen
  printf("\n Initialize Touch Driver...                   " );
  CHECK_HANDLE( EwBspTouchInit( EwScreenSize.X, EwScreenSize.Y, &DisplayInfo ));
*/

  #if EW_MEMORY_POOL_SIZE > 0
    // initialize heap manager
    printf("\nInitialize Memory Manager...             " );
    EwInitHeap( 0 );
    EwAddHeapMemoryPool( (void*)EW_MEMORY_POOL_ADDR, EW_MEMORY_POOL_SIZE );

    #if EW_EXTRA_POOL_SIZE > 0
      EwAddHeapMemoryPool( (void*)EW_EXTRA_POOL_ADDR, EW_EXTRA_POOL_SIZE );
    #endif

    printf("[OK]\n" );
  #endif

  // initialize the Graphics Engine and Runtime Environment
    printf("\nInitialize Graphics Engine...                " );
   CHECK_HANDLE( EwInitGraphicsEngine( 0 ));

  //create the applications root object ...
  printf("\nCreate Embedded Wizard Root Object...        " );
  RootObject = (CoreRoot)EwNewObjectIndirect( EwApplicationClass, 0 );
  CHECK_HANDLE( RootObject );

  EwLockObject( RootObject );
  CoreRoot__Initialize( RootObject, EwScreenSize );

  //create Embedded Wizard viewport object to provide uniform access to the framebuffer
  printf("\nCreate Embedded Wizard Viewport...           " );
  Viewport = EwInitViewport( EwScreenSize, EwNewRect( 0, 0, DisplayInfo.BufferWidth, DisplayInfo.BufferHeight ),
    0, 255, DisplayInfo.FrameBuffer, DisplayInfo.DoubleBuffer, 0, 0 );
  CHECK_HANDLE( Viewport );

  // initialize your device driver(s) that provide data for your GUI
  DeviceDriver_Initialize();

  printf("\nInit of Embedded Wizard Ends...\n" );
  return 1;
}

int EwProcess( void )
{

  int          timers  = 0;
  int          signals = 0;
  int          events  = 0;
  int          devices = 0;		// 0;	//VM
  XEnum        cmd     = CoreKeyCodeNoKey;
  int          noOfTouch;
  //XTouchEvent* touchEvent;
  int          touch;
  int          finger;
  XPoint       touchPos;

  // process data of your device driver(s) and update the GUI     application by setting properties or by triggering events
  devices = DeviceDriver_ProcessData();

  // receive keyboard inputs
  cmd = EwGetKeyCommand();

  if ( cmd != CoreKeyCodeNoKey )
  {
    if ( cmd == CoreKeyCodePower )
      return 0;

    // feed the application with a 'press' and 'release' event
    events |= CoreRoot__DriveKeyboardHitting( RootObject, cmd, 0, 1 );
    events |= CoreRoot__DriveKeyboardHitting( RootObject, cmd, 0, 0 );
  }
  /*
  // receive (multi-) touch inputs and provide it to the application
  noOfTouch = EwBspTouchGetEvents( &touchEvent );

  if ( noOfTouch > 0 )
  {
    for ( touch = 0; touch < noOfTouch; touch++ )
    {
      // get data out of the touch event
      finger     = touchEvent[ touch ].Finger;
      touchPos.X = touchEvent[ touch ].XPos;
      touchPos.Y = touchEvent[ touch ].YPos;

      // begin of touch cycle
      if ( touchEvent[ touch ].State == EW_BSP_TOUCH_DOWN )
        events |= CoreRoot__DriveMultiTouchHitting(  RootObject, 1, finger, touchPos );

      // movement during touch cycle
      else if ( touchEvent[ touch ].State == EW_BSP_TOUCH_MOVE )
        events |= CoreRoot__DriveMultiTouchMovement( RootObject, finger, touchPos );

      // end of touch cycle
      else if ( touchEvent[ touch ].State == EW_BSP_TOUCH_UP )
        events |= CoreRoot__DriveMultiTouchHitting(  RootObject, 0, finger, touchPos );
    }
  }
*/
  // process expired timers
  timers = EwProcessTimers();

  // process the pending signals
  signals = EwProcessSignals();

  // refresh the screen, if something has changed and draw its content
  if ( devices || timers || signals || events )
  {

    if ( CoreRoot__DoesNeedUpdate( RootObject ))
      EwUpdate( Viewport, RootObject );

    // just for debugging purposes: check the memory structure

    EwVerifyHeap();

    // after each processed message start the garbage collection
   EwReclaimMemory();

    // print current memory statistic to console interface
    #ifdef EW_PRINT_MEMORY_USAGE
      EwPrintProfilerStatistic( 0 );
    #endif

    // evaluate memory pools and print report
    #ifdef EW_DUMP_HEAP
      //EwDumpHeap( 1 );
    #endif
  }
//  else
//  {
    // otherwise sleep/suspend the UI application until a certain event occurs or a timer expires...
//    EwBspEventWait( EwNextTimerExpiration());
 // }

	_time_delay(20);
	//printf("\n EwProcess of Embedded Wizard Ends...\n" );
  return 1;
}

/****************************************** ewmain.c *************************************/
void EwPrintSystemInfo( void )
{
	printf( "---------------------------------------------\n" );
	printf( "Target system                                %s      \n", EW_PLATFORM_STRING );
	printf( "Color format                                 %s      \n", EW_FRAME_BUFFER_COLOR_FORMAT_STRING );
  #if EW_MEMORY_POOL_SIZE > 0
	printf( "MemoryPool address                           0x%08X  \n", EW_MEMORY_POOL_ADDR );
	printf( "MemoryPool size                              %u bytes\n", EW_MEMORY_POOL_SIZE );
  #endif
  #if EW_EXTRA_POOL_SIZE > 0
	printf( "ExtraPool address                            0x%08X  \n", EW_EXTRA_POOL_ADDR );
	printf( "ExtraPool size                               %u bytes\n", EW_EXTRA_POOL_SIZE );
  #endif
  #if EW_USE_SCRATCHPAD_BUFFER == 1
	printf( "Scratch-pad buffer address                   0x%08X  \n", DisplayInfo.FrameBuffer );
	printf( "Scratch-pad buffer size                      %u pixel\n", DisplayInfo.BufferWidth * DisplayInfo.BufferHeight );
  #else
	printf( "Framebuffer address                          0x%08X  \n", DisplayInfo.FrameBuffer );
  #endif
  #if EW_USE_DOUBLE_BUFFER == 1
	printf( "Doublebuffer address                         0x%08X  \n", DisplayInfo.DoubleBuffer );
  #endif
  #if EW_USE_OFFSCREEN_BUFFER == 1
  EwPrint( "Off-screen buffer                            used    \n" );
  #endif
  printf( "Display size                                 %u x %u \n", DisplayInfo.DisplayWidth, DisplayInfo.DisplayHeight );
  printf( "Framebuffer size                             %u x %u \n", DisplayInfo.BufferWidth, DisplayInfo.BufferHeight );
  printf( "EwScreenSize                                 %d x %d \n", EwScreenSize.X, EwScreenSize.Y );
  printf( "Graphics accelerator                         %s      \n", GRAPHICS_ACCELERATOR_STRING );
  printf( "Warp function support                        %s      \n", WARP_FUNCTION_SUPPORT_STRING );
  printf( "Vector graphics support                      %s      \n", VECTOR_GRAPHICS_SUPPORT_STRING );
  printf( "Bidirectional text support                   %s      \n", BIDI_TEXT_SUPPORT_STRING );
  printf( "Gradients support                            %s      \n", GRADIENTS_SUPPORT_STRING );
  printf( "Compression support                          %s      \n", COMPRESSION_SUPPORT_STRING );
  printf( "Index8 bitmap resource format                %s      \n", INDEX8_SURFACE_SUPPORT_STRING );
  printf( "RGB565 bitmap resource format                %s      \n", RGB565_SURFACE_SUPPORT_STRING );
  printf( "Native bitmap resource format                %s      \n", NATIVE_SURFACE_SUPPORT_STRING );
  printf( "Native destination bitmap support            %s      \n", NATIVE_DESTINATION_SURFACE_SUPPORT_STRING );
  printf( "Operating system                             %s      \n", OPERATING_SYSTEM_STRING );
  printf( "External Flash memory                        %s      \n", EXTERNAL_FLASH_STRING );
  #ifdef EW_BITMAP_PIXEL_SECTION_NAME
  printf( "Linker section for bitmap pixel              %s      \n", EW_STRINGIZE( EW_BITMAP_PIXEL_SECTION_NAME ));
  #endif
  #ifdef EW_FONT_PIXEL_SECTION_NAME
  printf( "Linker section for font pixel                %s      \n", EW_STRINGIZE( EW_FONT_PIXEL_SECTION_NAME ));
  #endif
  #ifdef EW_FONT_DATA_SECTION_NAME
  printf( "Linker section for font metrics              %s      \n", EW_STRINGIZE( EW_FONT_DATA_SECTION_NAME ));
  #endif
  #ifdef EW_CONST_STRING_SECTION_NAME
  printf( "Linker section for string constants          %s      \n", EW_STRINGIZE( EW_CONST_STRING_SECTION_NAME ));
  #endif
  printf( "Toolchain                                    %s      \n", TOOLCHAIN_STRING );
  #ifdef COMPILER_VERSION_STRING
 printf( "C-Compiler version                           %s      \n", COMPILER_VERSION_STRING );
  #endif
  printf( "Build date and time                          %s, %s  \n", __DATE__, __TIME__ );
  printf( "Runtime Environment (RTE) version            %u.%02u \n", EW_RTE_VERSION >> 16, EW_RTE_VERSION & 0xFF );
  printf( "Graphics Engine (GFX) version                %u.%02u \n", EW_GFX_VERSION >> 16, EW_GFX_VERSION & 0xFF );
  printf( "Max surface cache size                       %u bytes\n", EW_MAX_SURFACE_CACHE_SIZE );
  printf( "Glyph cache size                             %u x %u \n", EW_MAX_GLYPH_SURFACE_WIDTH, EW_MAX_GLYPH_SURFACE_HEIGHT );
  printf( "Max issue tasks                              %u      \n", EW_MAX_ISSUE_TASKS );
  printf( "Surface rotation                             %u      \n", EW_SURFACE_ROTATION );
  printf( "---------------------------------------------\n" );
}

void EwDone( void )
{
	// deinitialize your device driver(s)
  DeviceDriver_Deinitialize();

  // destroy the applications root object and release unused resources and memory
  printf("\n Shutting down Application...                 " );
  EwDoneViewport( Viewport );
  EwUnlockObject( RootObject );
  EwReclaimMemory();
  printf("[OK]\n" );

  // deinitialize the Graphics Engine
  printf("\n Deinitialize Graphics Engine...              " );
  EwDoneGraphicsEngine();
  printf("[OK]\n" );

  #if EW_MEMORY_POOL_SIZE > 0
  // deinitialize heap manager
    EwDoneHeap();
  #endif
    /*
  printf("\n Deinitialize Touch Driver...                 " );
  EwBspTouchDone();
  printf("\n [OK]\n" );
 */
  // deinitialize display
  EwBspDisplayDone( &DisplayInfo );

  Viewport   = 0;
  RootObject = 0;

printf("\n EwDone of Embedded Wizard Ends...\n" );
}

void GuiThread()
{
  // initialize Embedded Wizard application
  if ( EwInit() == 0 )
  {
	  _task_block();
  }
  EwPrintSystemInfo();
   // process the Embedded Wizard main loop
  while( EwProcess());

  // de-initialize Embedded Wizard application
  EwDone();
  _task_block();

}
/********************************************************************************
VM*/
