/*
 * ew_main.h
 *
 *  Created on: 21-Jul-2022
 *      Author: vmatai
 */

#ifndef SOURCE_CONTROLLER_TASKS_HEADERS_EW_MAIN_H_
#define SOURCE_CONTROLLER_TASKS_HEADERS_EW_MAIN_H_

#include <Common_defines.h>
#include "fsl_elcdif.h"
#include "ewrte.h"
#include "fsl_pxp.h"



// ****************************** ewmain.h ********************************************
#if defined __ICCARM__
  #define TOOLCHAIN_STRING "IAR Embedded Workbench"
  #define COMPILER_VERSION_STRING EW_STRINGIZE(__VER__)
#elif defined __CC_ARM || (__ARMCC_VERSION >= 6000000)
  #define TOOLCHAIN_STRING "Keil MDK"
  #define COMPILER_VERSION_STRING EW_STRINGIZE(__ARMCC_VERSION)
#elif defined __GNUC__
  #define TOOLCHAIN_STRING "GCC"
 // #define COMPILER_VERSION_STRING EW_STRINGIZE(__GNUC__) "."     EW_STRINGIZE(__GNUC_MINOR__) "." EW_STRINGIZE(__GNUC_PATCHLEVEL__)
#else
  #define TOOLCHAIN_STRING "unknown"
#endif

#ifdef EW_DONT_USE_WARP_FUNCTIONS
  #define WARP_FUNCTION_SUPPORT_STRING "disabled"
#else
  #define WARP_FUNCTION_SUPPORT_STRING "enabled"
#endif

#ifdef EW_DONT_USE_PATH_FUNCTIONS
  #define VECTOR_GRAPHICS_SUPPORT_STRING "disabled"
#else
  #define VECTOR_GRAPHICS_SUPPORT_STRING "enabled"
#endif

#ifdef EW_DONT_USE_BIDI_FUNCTIONS
  #define BIDI_TEXT_SUPPORT_STRING "disabled"
#else
  #define BIDI_TEXT_SUPPORT_STRING "enabled"
#endif

#ifdef EW_DONT_USE_GRADIENTS
  #define GRADIENTS_SUPPORT_STRING "disabled"
#else
  #define GRADIENTS_SUPPORT_STRING "enabled"
#endif

#ifdef EW_DONT_USE_COMPRESSION
  #define COMPRESSION_SUPPORT_STRING "disabled"
#else
  #define COMPRESSION_SUPPORT_STRING "enabled"
#endif

#ifdef EW_DONT_USE_INDEX8_SURFACES
  #define INDEX8_SURFACE_SUPPORT_STRING "disabled"
#else
  #define INDEX8_SURFACE_SUPPORT_STRING "enabled"
#endif

#ifdef EW_DONT_USE_RGB565_SURFACES
  #define RGB565_SURFACE_SUPPORT_STRING "disabled"
#else
  #define RGB565_SURFACE_SUPPORT_STRING "enabled"
#endif

#ifdef EW_DONT_USE_NATIVE_SURFACES
  #define NATIVE_SURFACE_SUPPORT_STRING "disabled"
#else
  #define NATIVE_SURFACE_SUPPORT_STRING "enabled"
#endif

#ifdef EW_DONT_USE_NATIVE_SURFACES_AS_DESTINATION
  #define NATIVE_DESTINATION_SURFACE_SUPPORT_STRING "disabled"
#else
  #define NATIVE_DESTINATION_SURFACE_SUPPORT_STRING "enabled"
#endif

#if EW_USE_GRAPHICS_ACCELERATOR == 1
  #define GRAPHICS_ACCELERATOR_STRING "PXP"
#else
  #define GRAPHICS_ACCELERATOR_STRING "none"
#endif

#if ( EW_USE_FREE_RTOS == 1 )
  #define OPERATING_SYSTEM_STRING "FreeRTOS"
#else
  #define OPERATING_SYSTEM_STRING "none"
#endif

  #define EXTERNAL_FLASH_STRING "Hyper Flash"

// ****************************** ew_bsp_console.h starts********************************************

void EwBspConsoleInit
(
  void
);

/*******************************************************************************
* FUNCTION:
*   EwBspConsolePutCharacter
*
* DESCRIPTION:
*   The function EwBspConsolePutCharacter sends the given character to the console
*   interface.
*
* ARGUMENTS:
*   aCharacter - The character to be send via the console interface.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspConsolePutCharacter
(
  unsigned char               aCharacter
);


/*******************************************************************************
* FUNCTION:
*   EwBspConsoleGetCharacter
*
* DESCRIPTION:
*   The function EwBspConsoleGetCharacter returns the current character from the
*   console interface. If no character is available within the input buffer, 0 is
*   returned.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   Current character from console input buffer or 0.
*
*******************************************************************************/
unsigned char* EwBspConsoleGetCharacter
(
  void
);


// ****************************** ew_bsp_display.h starts********************************************
typedef struct
{
  void*             FrameBuffer;
  void*             DoubleBuffer;
  int               BufferWidth;
  int               BufferHeight;
  int               DisplayWidth;
  int               DisplayHeight;
  int               UpdateMode;
  void*             Context;
  void*             MemoryPool;
} XDisplayInfo;

// flag to indicate normal display update with full access to frame buffer
#define EW_BSP_DISPLAY_UPDATE_NORMAL         0x00000000
// flag to indicate partial frame buffer update in case of a synchroneous single  buffer - update is divided in stripes (fields) defined by the display driver
#define EW_BSP_DISPLAY_UPDATE_PARTIAL        0x00000001
/* flag to indicate display update by using a scratch-pad buffer - update is done
   in subareas that fit into the scratch-pad buffer */
#define EW_BSP_DISPLAY_UPDATE_SCRATCHPAD     0x00000002

int EwBspDisplayInit( int aGuiWidth, int aGuiHeight, XDisplayInfo* aDisplayInfo );
void EwBspDisplayDone( XDisplayInfo* aDisplayInfo );
//int EwBspDisplayGetUpdateArea(  XRect* aUpdateRect);
void EwBspDisplayWaitForCompletion(void);
void EwBspDisplayCommitBuffer(void *aAddress,int aX,int aY,int aWidth,int aHeight);
void EwBspDisplaySetClut(unsigned long* aClut);
void updateProg_Counter(float xc, float yc);

// Macros for the LCD controller.
#define LCD_HSW 41
#define LCD_HFP 4
#define LCD_HBP 8
#define LCD_VSW 10
#define LCD_VFP 4
#define LCD_VBP 2
#define LCD_POL_FLAGS \
    (kELCDIF_DataEnableActiveHigh | kELCDIF_VsyncActiveLow | kELCDIF_HsyncActiveLow | kELCDIF_DriveDataOnRisingClkEdge)

// ****************************** ew_bsp_display.h ends ********************************************

// ****************************** ew_bsp_clock.h starts ********************************************
#if EW_CPU_LOAD_MEASURING == 1
  #define CPU_LOAD_SET_IDLE()           EwBspClockCpuLoadSetIdle()
  #define CPU_LOAD_SET_ACTIVE()         EwBspClockCpuLoadSetActive()
#else
  #define CPU_LOAD_SET_IDLE()
  #define CPU_LOAD_SET_ACTIVE()
#endif

unsigned long EwBspClockGetTicks(   void );
void EwBspClockTickIncrement(  void);

// ****************************** ew_bsp_graphics.h starts********************************************
#ifndef EW_USE_PXP_INTERRUPT_MODE
  #define EW_USE_PXP_INTERRUPT_MODE 1
#endif

#if EW_USE_PXP_INTERRUPT_MODE == 0
  #undef EW_USE_PXP_INTERRUPT_MODE
#endif

int EwBspGraphicsInit( void );
void EwBspGraphicsDone( void );
void EwBspGraphicsWaitForCompletion( void );
void EwBspGraphicsConcurrentOperation
(
  int               aEnable
);
void EwBspGraphicsFill
(
  uint32_t          aDstAddr,
  uint32_t          aDstPitch,
  uint32_t          aWidth,
  uint32_t          aHeight,
  uint32_t          aDstColorMode,
  uint32_t          aSrcColor
);
void EwBspGraphicsCopy
(
  uint32_t          aDstAddr,
  uint32_t          aSrcAddr,
  uint32_t          aDstPitch,
  uint32_t          aSrcPitch,
  uint32_t          aWidth,
  uint32_t          aHeight,
  uint32_t          aDstColorMode,
  uint32_t          aSrcColorMode,
  uint32_t          aSrcColor
);
void EwBspGraphicsCopyBlend
(
  uint32_t          aDstAddr,
  uint32_t          aSrcAddr,
  uint32_t          aDstPitch,
  uint32_t          aSrcPitch,
  uint32_t          aWidth,
  uint32_t          aHeight,
  uint32_t          aDstColorMode,
  uint32_t          aSrcColorMode,
  uint32_t          aSrcColor
);
// ****************************** ew_bsp_graphics.h ends ********************************************

// ****************************** ew_def.h starts********************************************

// defines for different framebuffer color formats
#define EW_FRAME_BUFFER_COLOR_FORMAT_Index8   1
#define EW_FRAME_BUFFER_COLOR_FORMAT_LumA44   2
#define EW_FRAME_BUFFER_COLOR_FORMAT_RGB565   3
#define EW_FRAME_BUFFER_COLOR_FORMAT_RGB888   4
#define EW_FRAME_BUFFER_COLOR_FORMAT_RGBA4444 5
#define EW_FRAME_BUFFER_COLOR_FORMAT_RGBA8888 6

# define EW_FRAME_BUFFER_COLOR_FORMAT EW_FRAME_BUFFER_COLOR_FORMAT_RGB565

#if ( EW_FRAME_BUFFER_COLOR_FORMAT == EW_FRAME_BUFFER_COLOR_FORMAT_RGBA8888 )
  #define EW_FRAME_BUFFER_COLOR_FORMAT_STRING "RGBA8888"
  #define EW_FRAME_BUFFER_DEPTH  4
#elif ( EW_FRAME_BUFFER_COLOR_FORMAT == EW_FRAME_BUFFER_COLOR_FORMAT_RGB888 )
  #define EW_FRAME_BUFFER_COLOR_FORMAT_STRING "RGB888"
  #define EW_FRAME_BUFFER_DEPTH  3
#elif ( EW_FRAME_BUFFER_COLOR_FORMAT == EW_FRAME_BUFFER_COLOR_FORMAT_RGBA4444 )
  #define EW_FRAME_BUFFER_COLOR_FORMAT_STRING "RGBA4444"
  #define EW_FRAME_BUFFER_DEPTH  2
#elif ( EW_FRAME_BUFFER_COLOR_FORMAT == EW_FRAME_BUFFER_COLOR_FORMAT_RGB565 )
  #define EW_FRAME_BUFFER_COLOR_FORMAT_STRING "RGB565"
  #define EW_FRAME_BUFFER_DEPTH  2
#elif ( EW_FRAME_BUFFER_COLOR_FORMAT == EW_FRAME_BUFFER_COLOR_FORMAT_Index8 )
  #define EW_FRAME_BUFFER_COLOR_FORMAT_STRING "Index8"
  #define EW_FRAME_BUFFER_DEPTH  1
#elif ( EW_FRAME_BUFFER_COLOR_FORMAT == EW_FRAME_BUFFER_COLOR_FORMAT_LumA44 )
  #define EW_FRAME_BUFFER_COLOR_FORMAT_STRING "LumA44"
  #define EW_FRAME_BUFFER_DEPTH  1
#else
	#error EW_FRAME_BUFFER_COLOR_FORMAT not supported
#endif


// ****************************** ewconfig.h *********************************************
#define EW_PLATFORM_STRING    "IMXRT1060-EVK"
//#define SDRAM_BASE_ADDR       0x81000000
#define SDRAM_SIZE_BYTES      (14 * 1024 * 1024)

#define EW_USE_TERMINAL_INPUT 0
#define EW_CPU_LOAD_MEASURING 1

#define EW_USE_DOUBLE_BUFFER            1

#define EW_DISPLAY_WIDTH                480
#define EW_DISPLAY_HEIGHT               272

#define EW_FRAME_BUFFER_WIDTH           480
#define EW_FRAME_BUFFER_HEIGHT          272


static XDisplayInfo DisplayInfo;

#define EW_FRAME_BUFFER_ADDR  DisplayInfo.FrameBuffer
// calculated addresses for framebuffer(s)
//#define EW_FRAME_BUFFER_ADDR  SDRAM_BASE_ADDR
#define EW_FRAME_BUFFER_SIZE  EW_FRAME_BUFFER_WIDTH * EW_FRAME_BUFFER_HEIGHT * EW_FRAME_BUFFER_DEPTH	//480*272*2 = 261,120

#if EW_USE_DOUBLE_BUFFER == 1

  //#define EW_DOUBLE_BUFFER_ADDR  (EW_FRAME_BUFFER_ADDR + EW_FRAME_BUFFER_SIZE)
  #define EW_DOUBLE_BUFFER_SIZE  EW_FRAME_BUFFER_SIZE

#else

  #define EW_DOUBLE_BUFFER_ADDR  0
  #define EW_DOUBLE_BUFFER_SIZE  0
  #define EW_NUMBER_OF_FIELDS    3

#endif

#define EW_UNUSED_ARG( aArg )                                                    \
  (void)( aArg )


#define EW_MEMORY_POOL_ADDR      DisplayInfo.MemoryPool
#define EW_MEMORY_POOL_SIZE      EW_FRAME_BUFFER_SIZE + EW_DOUBLE_BUFFER_SIZE
//#define EW_MEMORY_POOL_ADDR      (EW_FRAME_BUFFER_ADDR + EW_FRAME_BUFFER_SIZE + EW_DOUBLE_BUFFER_SIZE)
//#define EW_MEMORY_POOL_SIZE      SDRAM_SIZE_BYTES - EW_FRAME_BUFFER_SIZE - EW_DOUBLE_BUFFER_SIZE

#define EW_EXTRA_POOL_ADDR       0
#define EW_EXTRA_POOL_SIZE       0

 #define EW_PRINT_MEMORY_USAGE
 #define EW_DUMP_HEAP

 #define EW_SUPPORT_GFX_TASK_TRACING
 #define EW_PRINT_GFX_TASKS
 #define EW_PRINT_PERF_COUNTERS
 #define EW_USE_IMMEDIATE_GARBAGE_COLLECTION

#define EW_MAX_STRING_CACHE_SIZE      0x4000
#define EW_MAX_SURFACE_CACHE_SIZE   0x800000
#define EW_MAX_GLYPH_SURFACE_WIDTH       256
#define EW_MAX_GLYPH_SURFACE_HEIGHT      256
#define EW_MAX_NO_OF_GLYPHS                0
#define EW_MAX_ISSUE_TASKS               100

#define EW_LAZY_LOAD_BITMAPS                              1
#define EW_LAZY_LOAD_BITMAPS_IF_ANIMATED_ONLY             1
#define EW_DISCARD_BITMAPS                                1
#define EW_DISCARD_BITMAPS_IF_ANIMATED_ONLY               1
#define EW_DISCARD_BITMAPS_IF_NOT_USED_IN_CURRENT_UPDATE  0
#define EW_DISCARD_BITMAPS_IF_NOT_USED_IN_RECENT_UPDATES  0
#define EW_CACHE_OFFSCREEN_SURFACES                       0
// ****************************** ew_main.c ************************************************

int EwInit( void );
int EwProcess( void );
void EwDone( void );
void GuiThread();


#endif /* SOURCE_CONTROLLER_TASKS_HEADERS_EW_MAIN_H_ */
